""" REST views for the workspace API
"""
from rest_framework import status
from rest_framework.decorators import api_view, permission_classes
from rest_framework.exceptions import ValidationError
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response
from rest_framework.views import APIView

import core_main_app.components.group.api as group_api
import core_main_app.components.user.api as user_api
import core_main_app.components.workspace.api as workspace_api
from core_main_app.access_control.exceptions import AccessControlError
from core_main_app.commons import exceptions
from core_main_app.rest.group.serializers import GroupSerializer
from core_main_app.rest.user.serializers import UserSerializer
from core_main_app.rest.workspace.serializers import WorkspaceSerializer


class WorkspaceList(APIView):
    """List all user Workspace, or create a new one"""

    permission_classes = (IsAuthenticated,)

    def get(self, request):
        """Get all user workspaces

        Args:

            request: HTTP request

        Returns:

            - code: 200
              content: List of workspace
            - code: 500
              content: Internal server error
        """
        try:
            if request.user.is_superuser:
                workspace_list = workspace_api.get_all()
            else:
                workspace_list = workspace_api.get_all_by_owner(request.user)

            # Serialize object
            serializer = WorkspaceSerializer(workspace_list, many=True)

            # Return response
            return Response(serializer.data, status=status.HTTP_200_OK)
        except Exception as api_exception:
            content = {"message": str(api_exception)}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)

    def post(self, request):
        """Create a Workspace

        Parameters:

            {
                "title": "document_title",
            }

        Args:

            request: HTTP request

        Returns:

            - code: 201
              content: Created workspace
            - code: 400
              content: Validation error / not unique / model error
            - code: 500
              content: Internal server error
        """
        try:
            # Build serializer
            serializer = WorkspaceSerializer(data=request.data)

            # Validate data
            serializer.is_valid(True)

            # Save data
            serializer.save(user=request.user)

            return Response(serializer.data, status=status.HTTP_201_CREATED)
        except ValidationError as validation_exception:
            content = {"message": validation_exception.detail}
            return Response(content, status=status.HTTP_400_BAD_REQUEST)
        except exceptions.ModelError as validation_exception:
            content = {"message": str(validation_exception)}
            return Response(content, status=status.HTTP_400_BAD_REQUEST)
        except exceptions.NotUniqueError as validation_exception:
            content = {"message": str(validation_exception)}
            return Response(content, status=status.HTTP_400_BAD_REQUEST)
        except KeyError as validation_exception:
            content = {"message": validation_exception}
            return Response(content, status=status.HTTP_400_BAD_REQUEST)
        except Exception as api_exception:
            content = {"message": str(api_exception)}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)


class WorkspaceDetail(APIView):
    """Workspace Detail"""

    permission_classes = (IsAuthenticated,)

    def get(self, request, pk):
        """Get Workspace from db

        Args:

            request: HTTP request
            pk: ObjectId

        Returns:

            Workspace
        """
        try:
            # Get object
            workspace_object = workspace_api.get_by_id(pk)

            # Serialize object
            serializer = WorkspaceSerializer(workspace_object)

            # Return response
            return Response(serializer.data)
        except exceptions.DoesNotExist:
            content = {"message": "Workspace not found."}
            return Response(content, status=status.HTTP_404_NOT_FOUND)
        except Exception as api_exception:
            content = {"message": str(api_exception)}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)

    def delete(self, request, pk):
        """Delete a Workspace

        Args:

            request: HTTP request
            pk: ObjectId

        Returns:

            - code: 204
              content: Deletion succeed
            - code: 403
              content: Authentication error
            - code: 404
              content: Object was not found
            - code: 500
              content: Internal server error
        """
        try:
            # Get object
            workspace_object = workspace_api.get_by_id(pk)

            # delete object
            workspace_api.delete(workspace_object, request.user)

            # Return response
            return Response(status=status.HTTP_204_NO_CONTENT)
        except exceptions.DoesNotExist:
            content = {"message": "Workspace not found."}
            return Response(content, status=status.HTTP_404_NOT_FOUND)
        except AccessControlError as ace:
            content = {"message": str(ace)}
            return Response(content, status=status.HTTP_403_FORBIDDEN)
        except Exception as api_exception:
            content = {"message": str(api_exception)}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)


@api_view(["GET"])
@permission_classes((IsAuthenticated,))
def get_workspaces_with_read_access(request):
    """Get all workspaces with read access

    Args:

        request: HTTP request

    Returns:

        - code: 200
          content: list of workspace
        - code: 500
          content: Internal server error
    """
    return _list_of_workspaces_to_response(
        workspace_api.get_all_workspaces_with_read_access_by_user(request.user)
    )


@api_view(["GET"])
@permission_classes((IsAuthenticated,))
def get_workspaces_with_write_access(request):
    """Get all workspaces with write access

    Args:

        request: HTTP request

    Returns:

        - code: 200
          content: list of workspace
        - code: 500
          content: Internal server error
    """
    return _list_of_workspaces_to_response(
        workspace_api.get_all_workspaces_with_write_access_by_user(request.user)
    )


def _list_of_workspaces_to_response(func):
    """Serialize and generate response the list of workspaces you can get with func method

    Args:

        func: function

    Returns:

        - code: 200
          content: list of workspace
        - code: 500
          content: Internal server error
    """
    try:
        # Get object list
        workspace_object_list = func

        # Serialize object
        return_value = WorkspaceSerializer(workspace_object_list, many=True)

        # Return response
        return Response(return_value.data, status=status.HTTP_200_OK)

    except Exception as api_exception:
        content = {"message": str(api_exception)}
        return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)


@api_view(["GET"])
@permission_classes((IsAuthenticated,))
def is_workspace_public(request, pk):
    """Is the workspace public

    Args:

        request: HTTP request
        pk: ObjectId

    Returns:

        - code: 200
          content: Boolean
        - code: 404
          content: Object was not found
        - code: 500
          content: Internal server error
    """
    try:
        # Get object
        workspace_object = workspace_api.get_by_id(pk)

        # Return response
        return Response(
            workspace_api.is_workspace_public(workspace_object),
            status=status.HTTP_200_OK,
        )
    except exceptions.DoesNotExist:
        content = {"message": "Workspace not found."}
        return Response(content, status=status.HTTP_404_NOT_FOUND)
    except Exception as api_exception:
        content = {"message": str(api_exception)}
        return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)


@api_view(["PATCH"])
@permission_classes((IsAuthenticated,))
def set_workspace_public(request, pk):
    """Set the workspace public

    Args:

        request: HTTP request
        pk: ObjectId

    Returns:

        - code: 200
          content: None
        - code: 403
          content: Authentication error
        - code: 404
          content: Object was not found
        - code: 500
          content: Internal server error
    """
    try:
        # Get object
        workspace_object = workspace_api.get_by_id(pk)

        # Set the workspace public
        workspace_api.set_workspace_public(workspace_object, request.user)

        # Return response
        return Response(status=status.HTTP_200_OK)
    except exceptions.DoesNotExist:
        content = {"message": "Workspace not found."}
        return Response(content, status=status.HTTP_404_NOT_FOUND)
    except AccessControlError as ace:
        content = {"message": str(ace)}
        return Response(content, status=status.HTTP_403_FORBIDDEN)
    except Exception as api_exception:
        content = {"message": str(api_exception)}
        return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)


@api_view(["PATCH"])
@permission_classes((IsAuthenticated,))
def set_workspace_private(request, pk):
    """Set the workspace private

    Args:

        request: HTTP request
        pk: ObjectId

    Returns:

        - code: 200
          content: None
        - code: 403
          content: Authentication error
        - code: 404
          content: Object was not found
        - code: 500
          content: Internal server error
    """
    try:
        # Get object
        workspace_object = workspace_api.get_by_id(pk)

        # Set the workspace public
        workspace_api.set_workspace_private(workspace_object, request.user)

        # Return response
        return Response(status=status.HTTP_200_OK)
    except exceptions.DoesNotExist:
        content = {"message": "Workspace not found."}
        return Response(content, status=status.HTTP_404_NOT_FOUND)
    except AccessControlError as ace:
        content = {"message": str(ace)}
        return Response(content, status=status.HTTP_403_FORBIDDEN)
    except Exception as api_exception:
        content = {"message": str(api_exception)}
        return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)


@api_view(["GET"])
@permission_classes((IsAuthenticated,))
def get_list_user_can_write_workspace(request, pk):
    """Get list of users that have write access to workspace

    Args:

        request: HTTP request

    Returns:

        - code: 200
          content: list of user
        - code: 403
          content: Authentication error
        - code: 404
          content: Object was not found
        - code: 500
          content: Internal server error
    """
    return _list_of_users_or_groups_to_response(
        pk,
        workspace_api.get_list_user_can_write_workspace,
        request.user,
        UserSerializer,
    )


@api_view(["GET"])
@permission_classes((IsAuthenticated,))
def get_list_user_can_read_workspace(request, pk):
    """Get list of users that have read access to workspace

    Args:

        request: HTTP request

    Returns:

        - code: 200
          content: list of user
        - code: 403
          content: Authentication error
        - code: 404
          content: Object was not found
        - code: 500
          content: Internal server error
    """
    return _list_of_users_or_groups_to_response(
        pk, workspace_api.get_list_user_can_read_workspace, request.user, UserSerializer
    )


@api_view(["GET"])
@permission_classes((IsAuthenticated,))
def get_list_user_can_access_workspace(request, pk):
    """Get list of users that have read or write access to workspace

    Args:

        request: HTTP request

    Returns:

        - code: 200
          content: list of user
        - code: 403
          content: Authentication error
        - code: 404
          content: Object was not found
        - code: 500
          content: Internal server error
    """
    return _list_of_users_or_groups_to_response(
        pk,
        workspace_api.get_list_user_can_access_workspace,
        request.user,
        UserSerializer,
    )


@api_view(["GET"])
@permission_classes((IsAuthenticated,))
def get_list_group_can_write_workspace(request, pk):
    """Get list of groups that have write access to workspace

    Args:

        request: HTTP request

    Returns:

        - code: 200
          content: list of group
        - code: 403
          content: Authentication error
        - code: 404
          content: Object was not found
        - code: 500
          content: Internal server error
    """
    return _list_of_users_or_groups_to_response(
        pk,
        workspace_api.get_list_group_can_write_workspace,
        request.user,
        GroupSerializer,
    )


@api_view(["GET"])
@permission_classes((IsAuthenticated,))
def get_list_group_can_read_workspace(request, pk):
    """Get list of groups that have read access to workspace

    Args:

        request: HTTP request

    Returns:

        - code: 200
          content: list of group
        - code: 403
          content: Authentication error
        - code: 404
          content: Object was not found
        - code: 500
          content: Internal server error
    """
    return _list_of_users_or_groups_to_response(
        pk,
        workspace_api.get_list_group_can_read_workspace,
        request.user,
        GroupSerializer,
    )


@api_view(["GET"])
@permission_classes((IsAuthenticated,))
def get_list_group_can_access_workspace(request, pk):
    """Get list of groups that have read or write access to workspace

    Args:

        request: HTTP request

    Returns:

        - code: 200
          content: list of group
        - code: 403
          content: Authentication error
        - code: 404
          content: Object was not found
        - code: 500
          content: Internal server error
    """
    return _list_of_users_or_groups_to_response(
        pk,
        workspace_api.get_list_group_can_access_workspace,
        request.user,
        GroupSerializer,
    )


def _list_of_users_or_groups_to_response(pk, func, user, serializer):
    """List of users or groups to response

    Args:

        pk: ObjectId
        func: function
        user: user

    Returns:

        - code: 200
          content: list of group or user
        - code: 403
          content: Authentication error
        - code: 404
          content: Object was not found
        - code: 500
          content: Internal server error
    """
    try:
        # Get object
        workspace_object = workspace_api.get_by_id(pk)

        # Get list User
        list_user = func(workspace_object, user)

        # Serialize object
        serializer = serializer(list_user, many=True)

        # Return response
        return Response(serializer.data, status=status.HTTP_200_OK)

    except exceptions.DoesNotExist:
        content = {"message": "Workspace not found."}
        return Response(content, status=status.HTTP_404_NOT_FOUND)
    except AccessControlError as ace:
        content = {"message": str(ace)}
        return Response(content, status=status.HTTP_403_FORBIDDEN)
    except Exception as api_exception:
        content = {"message": str(api_exception)}
        return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)


def _add_or_remove_to_user_or_group_right_to_workspace(
    request, pk, user_or_group_id, func, get_group_or_user_func
):
    """Add or remove a right to a user or a group

    Args:

        request: HTTP request
        pk: ObjectId
        user_or_group_id: ObjectId
        func: function
        get_group_or_user_func: function

    Returns:

        - code: 200
          content: None
        - code: 403
          content: Authentication error
        - code: 404
          content: Object was not found
        - code: 500
          content: Internal server error
    """
    try:
        # Get object
        workspace_object = workspace_api.get_by_id(pk)

        # Get user
        user_or_group = get_group_or_user_func(user_or_group_id)

        # Add right
        func(workspace_object, user_or_group, request.user)

        # Return response
        return Response(status=status.HTTP_200_OK)

    except exceptions.DoesNotExist:
        content = {"message": "Workspace not found."}
        return Response(content, status=status.HTTP_404_NOT_FOUND)
    except AccessControlError as ace:
        content = {"message": str(ace)}
        return Response(content, status=status.HTTP_403_FORBIDDEN)
    except Exception as api_exception:
        content = {"message": str(api_exception)}
        return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)


@api_view(["PATCH"])
@permission_classes((IsAuthenticated,))
def add_user_read_right_to_workspace(request, pk, user_id):
    """Add to the user the read right to the Workspace

    Args:

        request: HTTP request
        pk: ObjectId
        user_id: ObjectId

    Returns:

        - code: 200
          content: None
        - code: 403
          content: Authentication error
        - code: 404
          content: Object was not found
        - code: 500
          content: Internal server error
    """
    return _add_or_remove_to_user_or_group_right_to_workspace(
        request,
        pk,
        user_id,
        workspace_api.add_user_read_access_to_workspace,
        user_api.get_user_by_id,
    )


@api_view(["PATCH"])
@permission_classes((IsAuthenticated,))
def add_user_write_right_to_workspace(request, pk, user_id):
    """Add to the user the write right to the workspace

    Args:

        request: HTTP request
        pk: ObjectId
        user_id: ObjectId

    Returns:

        - code: 200
          content: None
        - code: 403
          content: Authentication error
        - code: 404
          content: Object was not found
        - code: 500
          content: Internal server error
    """
    return _add_or_remove_to_user_or_group_right_to_workspace(
        request,
        pk,
        user_id,
        workspace_api.add_user_write_access_to_workspace,
        user_api.get_user_by_id,
    )


@api_view(["PATCH"])
@permission_classes((IsAuthenticated,))
def add_group_read_right_to_workspace(request, pk, group_id):
    """Add to the group the read right to the workspace

    Args:

        request: HTTP request
        pk: ObjectId
        group_id: ObjectId

    Returns:

        - code: 200
          content: None
        - code: 403
          content: Authentication error
        - code: 404
          content: Object was not found
        - code: 500
          content: Internal server error
    """
    return _add_or_remove_to_user_or_group_right_to_workspace(
        request,
        pk,
        group_id,
        workspace_api.add_group_read_access_to_workspace,
        group_api.get_group_by_id,
    )


@api_view(["PATCH"])
@permission_classes((IsAuthenticated,))
def add_group_write_right_to_workspace(request, pk, group_id):
    """Add to the group the write right to the workspace

    Args:

        request: HTTP request
        pk: ObjectId
        group_id: ObjectId

    Returns:

        - code: 200
          content: None
        - code: 403
          content: Authentication error
        - code: 404
          content: Object was not found
        - code: 500
          content: Internal server error
    """
    return _add_or_remove_to_user_or_group_right_to_workspace(
        request,
        pk,
        group_id,
        workspace_api.add_group_write_access_to_workspace,
        group_api.get_group_by_id,
    )


@api_view(["PATCH"])
@permission_classes((IsAuthenticated,))
def remove_user_read_right_to_workspace(request, pk, user_id):
    """Remove from the user the read right to the workspace

    Args:

        request: HTTP request
        pk: ObjectId
        user_id: ObjectId

    Returns:

        - code: 200
          content: None
        - code: 403
          content: Authentication error
        - code: 404
          content: Object was not found
        - code: 500
          content: Internal server error
    """
    return _add_or_remove_to_user_or_group_right_to_workspace(
        request,
        pk,
        user_id,
        workspace_api.remove_user_read_access_to_workspace,
        user_api.get_user_by_id,
    )


@api_view(["PATCH"])
@permission_classes((IsAuthenticated,))
def remove_user_write_right_to_workspace(request, pk, user_id):
    """Remove from the user the write right to the workspace

    Args:

        request: HTTP request
        pk: ObjectId
        user_id: ObjectId

    Returns:

        - code: 200
          content: None
        - code: 403
          content: Authentication error
        - code: 404
          content: Object was not found
        - code: 500
          content: Internal server error
    """
    return _add_or_remove_to_user_or_group_right_to_workspace(
        request,
        pk,
        user_id,
        workspace_api.remove_user_write_access_to_workspace,
        user_api.get_user_by_id,
    )


@api_view(["PATCH"])
@permission_classes((IsAuthenticated,))
def remove_group_read_right_to_workspace(request, pk, group_id):
    """Remove from the group the read right to the workspace

    Args:

        request: HTTP request
        pk: ObjectId
        group_id: ObjectId

    Returns:

        - code: 200
          content: None
        - code: 403
          content: Authentication error
        - code: 404
          content: Object was not found
        - code: 500
          content: Internal server error
    """
    return _add_or_remove_to_user_or_group_right_to_workspace(
        request,
        pk,
        group_id,
        workspace_api.remove_group_read_access_to_workspace,
        group_api.get_group_by_id,
    )


@api_view(["PATCH"])
@permission_classes((IsAuthenticated,))
def remove_group_write_right_to_workspace(request, pk, group_id):
    """Remove from the group the write right to the workspace

    Args:

        request: HTTP request
        pk: ObjectId
        group_id: ObjectId

    Returns:

        - code: 200
          content: None
        - code: 403
          content: Authentication error
        - code: 404
          content: Object was not found
        - code: 500
          content: Internal server error
    """
    return _add_or_remove_to_user_or_group_right_to_workspace(
        request,
        pk,
        group_id,
        workspace_api.remove_group_write_access_to_workspace,
        group_api.get_group_by_id,
    )



""" Unit Test Data
"""
from types import SimpleNamespace

from bson.objectid import ObjectId
import datetime
import pytz
from mock.mock import patch

from core_main_app.commons import exceptions
from core_main_app.components.data.api import check_xml_file_is_valid
from core_main_app.components.data.models import Data
from core_main_app.components.data import api as data_api
from core_main_app.system import api as system_api
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.integration_tests.integration_base_test_case import (
    MongoIntegrationBaseTestCase,
)
from core_main_app.settings import DATA_SORTING_FIELDS
from tests.components.data.fixtures.fixtures import (
    DataFixtures,
    AccessControlDataFixture,
)
from core_main_app.components.data import tasks as data_task
from core_main_app.components.template import api as template_api
from core_main_app.utils.integration_tests.integration_base_transaction_test_case import (
    MongoIntegrationTransactionTestCase,
)
from tests.components.user.fixtures.fixtures import UserFixtures
from tests.components.data.fixtures.fixtures import DataMigrationFixture

fixture_data_template = DataMigrationFixture()
fixture_data = DataFixtures()
access_control_data_fixture = AccessControlDataFixture()


class TestDataGetAll(MongoIntegrationBaseTestCase):

    fixture = access_control_data_fixture

    def test_data_get_all_return_collection_of_data(self):
        # Act
        result = Data.get_all(DATA_SORTING_FIELDS)
        # Assert
        self.assertTrue(all(isinstance(item, Data) for item in result))

    def test_data_get_all_return_objects_data_in_collection(self):
        # Act
        result = Data.get_all(DATA_SORTING_FIELDS)
        # Assert
        self.assertTrue(len(self.fixture.data_collection) == result.count())

    def test_data_get_all_ordering(self):
        # Arrange
        access_control_data_fixture.insert_data()
        ascending_order_by_field = ["+title"]
        descending_order_by_field = ["-title"]
        # Act
        ascending_result = Data.get_all(ascending_order_by_field)
        descending_result = Data.get_all(descending_order_by_field)
        # Assert
        for i in range(len(ascending_result)):
            self.assertTrue(
                ascending_result.all()[i].title
                == descending_result.all()[len(ascending_result) - i - 1].title
            )

    def test_data_get_all_ascending_sorting(self):
        # Arrange
        ascending_order_by_field = ["+title"]
        # Act
        ascending_result = Data.get_all(ascending_order_by_field)
        # Assert
        self.assertTrue(self.fixture.data_1.title == ascending_result.all()[0].title)
        self.assertTrue(self.fixture.data_2.title == ascending_result.all()[1].title)

    def test_data_get_all_descending_sorting(self):
        # Arrange
        descending_order_by_field = ["-title"]
        # Act
        descending_result = Data.get_all(descending_order_by_field)
        # Assert
        self.assertTrue(
            self.fixture.data_2.title
            == descending_result.all()[len(descending_result) - 2].title
        )
        self.assertTrue(
            self.fixture.data_1.title
            == descending_result.all()[len(descending_result) - 1].title
        )

    def test_data_get_all_multi_field_sorting(self):
        # Arrange
        ascending_order_by_multi_field = ["+title", "+user_id"]
        descending_order_by_multi_field = ["+title", "-user_id"]
        # Act
        ascending_result = Data.get_all(ascending_order_by_multi_field)
        descending_result = Data.get_all(descending_order_by_multi_field)
        # Assert
        self.assertEqual(self.fixture.data_4.user_id, ascending_result.all()[4].user_id)
        self.assertEqual(self.fixture.data_5.user_id, ascending_result.all()[3].user_id)

        self.assertEqual(
            self.fixture.data_4.user_id, descending_result.all()[3].user_id
        )
        self.assertEqual(
            self.fixture.data_5.user_id, descending_result.all()[4].user_id
        )

    def test_data_get_all_api_without_sorting_param_use_default_data_sorting_setting(
        self,
    ):
        # Arrange
        mock_user = _create_user("1", is_superuser=True)
        # Act
        data = data_api.get_all(mock_user)
        # Assert
        self.assertListEqual(list(data), self.fixture.data_collection)


class TestDataGetAllExcept(MongoIntegrationBaseTestCase):

    fixture = access_control_data_fixture

    def test_data_get_all_except_return_collection_of_data(self):
        # Act
        db_content = Data.get_all(DATA_SORTING_FIELDS)
        excluded_id_list = [str(db_content[0].pk)]

        result = Data.get_all_except([], excluded_id_list)
        # Assert
        self.assertTrue(all(isinstance(item, Data) for item in result))

    def test_data_get_all_return_objects_data_in_collection(self):
        # Act
        db_content = Data.get_all(DATA_SORTING_FIELDS)
        excluded_id_list = [str(db_content[0].pk)]

        result = Data.get_all_except([], excluded_id_list)
        # Assert
        self.assertTrue(
            result.count() == len(self.fixture.data_collection) - len(excluded_id_list)
        )

    def test_data_get_all_except_empty_list_return_collection_of_data(self):
        # Act
        result = Data.get_all_except(DATA_SORTING_FIELDS, id_list=[])
        # Assert
        self.assertTrue(all(isinstance(item, Data) for item in result))

    def test_data_get_all_except_empty_list_return_objects_data_in_collection(self):
        # Act
        result = Data.get_all_except(DATA_SORTING_FIELDS, id_list=[])
        # Assert
        self.assertTrue(result.count() == len(self.fixture.data_collection))

    def test_data_get_all_except_inexistant_id_return_collection_of_data(self):
        # Act
        object_id_list = [str(data.pk) for data in Data.get_all(DATA_SORTING_FIELDS)]
        inexistant_object_id = str(ObjectId())

        # If the generated object id correspond to one in DB we generate another one
        while inexistant_object_id in object_id_list:
            inexistant_object_id = str(ObjectId)

        excluded_id_list = [inexistant_object_id]

        result = Data.get_all_except([], excluded_id_list)
        # Assert
        self.assertTrue(all(isinstance(item, Data) for item in result))

    def test_data_get_all_except_inexistant_id_return_objects_data_in_collection(self):
        # Act
        object_id_list = [str(data.pk) for data in Data.get_all(DATA_SORTING_FIELDS)]
        inexistant_object_id = str(ObjectId())

        # If the generated object id correspond to one in DB we generate another one
        while inexistant_object_id in object_id_list:
            inexistant_object_id = str(ObjectId)

        excluded_id_list = [inexistant_object_id]

        result = Data.get_all_except([], excluded_id_list)
        # Assert
        self.assertTrue(result.count() == len(self.fixture.data_collection))

    def test_data_get_all_except_no_params_return_collection_of_data(self):
        # Act
        result = Data.get_all_except(DATA_SORTING_FIELDS)
        # Assert
        self.assertTrue(all(isinstance(item, Data) for item in result))

    def test_data_get_all_except_no_params_return_objects_data_in_collection(self):
        # Act
        result = Data.get_all_except(DATA_SORTING_FIELDS)
        # Assert
        self.assertTrue(result.count() == len(self.fixture.data_collection))

    def test_get_all_except_data_ordering(self):
        # Arrange
        ascending_order_by_field = ["+title"]
        descending_order_by_field = ["-title"]
        # Act
        ascending_result = Data.get_all_except(ascending_order_by_field)
        descending_result = Data.get_all_except(descending_order_by_field)
        # Assert
        for i in range(len(ascending_result)):
            self.assertTrue(
                ascending_result.all()[i].title
                == descending_result.all()[len(ascending_result) - i - 1].title
            )

    def test_get_all_except_data_ascending_sorting(self):
        # Arrange
        ascending_order_by_field = ["+title"]
        # Act
        ascending_result = Data.get_all_except(ascending_order_by_field)
        # Assert
        self.assertTrue(self.fixture.data_1.title == ascending_result.all()[0].title)
        self.assertTrue(self.fixture.data_2.title == ascending_result.all()[1].title)

    def test_get_all_except_data_descending_sorting(self):
        # Arrange
        descending_order_by_field = ["-title"]
        # Act
        descending_result = Data.get_all_except(descending_order_by_field)
        # Assert
        self.assertTrue(
            self.fixture.data_2.title
            == descending_result.all()[len(descending_result) - 2].title
        )
        self.assertTrue(
            self.fixture.data_1.title
            == descending_result.all()[len(descending_result) - 1].title
        )

    def test_get_all_except_data_multi_field_sorting(self):
        # Arrange
        ascending_order_by_multi_field = ["+title", "+user_id"]
        descending_order_by_multi_field = ["+title", "-user_id"]
        # Act
        ascending_result = Data.get_all_except(ascending_order_by_multi_field)
        descending_result = Data.get_all_except(descending_order_by_multi_field)
        # Assert
        self.assertEqual(self.fixture.data_4.user_id, ascending_result.all()[4].user_id)
        self.assertEqual(self.fixture.data_5.user_id, ascending_result.all()[3].user_id)

        self.assertEqual(
            self.fixture.data_4.user_id, descending_result.all()[3].user_id
        )
        self.assertEqual(
            self.fixture.data_5.user_id, descending_result.all()[4].user_id
        )


class TestDataGetById(MongoIntegrationBaseTestCase):

    fixture = fixture_data

    def test_data_get_by_id_raises_api_error_if_not_found(self):
        # Act # Assert
        with self.assertRaises(exceptions.DoesNotExist):
            Data.get_by_id(ObjectId())

    def test_data_get_by_id_return_data_if_found(self):
        # Act
        result = Data.get_by_id(self.fixture.data_1.id)
        # Assert
        self.assertEqual(result, self.fixture.data_1)


class TestDataGetAllByUserId(MongoIntegrationBaseTestCase):

    fixture = access_control_data_fixture

    def test_data_get_all_by_user_id_return_collection_of_data_from_user(self):
        # Arrange
        user_id = 1
        # Act
        result = Data.get_all_by_user_id(user_id, DATA_SORTING_FIELDS)
        # Assert
        self.assertTrue(all(item.user_id == str(user_id) for item in result))

    def test_data_get_all_by_user_id_return_empty_collection_of_data_from_user_does_not_exist(
        self,
    ):
        # Arrange
        user_id = 800
        # Act
        result = Data.get_all_by_user_id(user_id, DATA_SORTING_FIELDS)
        # Assert
        self.assertTrue(result.count() == 0)

    def test_get_all_by_user_id_data_ordering(self):
        # Arrange
        user_id = 1
        ascending_order_by_field = ["+title"]
        descending_order_by_field = ["-title"]
        # Act
        ascending_result = Data.get_all_by_user_id(user_id, ascending_order_by_field)
        descending_result = Data.get_all_by_user_id(user_id, descending_order_by_field)
        # Assert
        for i in range(len(ascending_result)):
            self.assertTrue(
                ascending_result.all()[i].title
                == descending_result.all()[len(ascending_result) - i - 1].title
            )

    def test_get_all_by_user_id_data_ascending_sorting(self):
        # Arrange
        ascending_order_by_field = ["+title"]
        user_id = 1
        # Act
        ascending_result = Data.get_all_by_user_id(user_id, ascending_order_by_field)
        # Assert
        self.assertTrue(self.fixture.data_1.title == ascending_result.all()[0].title)
        self.assertTrue(self.fixture.data_3.title == ascending_result.all()[1].title)

    def test_get_all_by_user_id_data_descending_sorting(self):
        # Arrange
        descending_order_by_field = ["-title"]
        user_id = 1
        # Act
        descending_result = Data.get_all_by_user_id(user_id, descending_order_by_field)
        # Assert
        self.assertTrue(
            self.fixture.data_3.title
            == descending_result.all()[len(descending_result) - 2].title
        )
        self.assertTrue(
            self.fixture.data_1.title
            == descending_result.all()[len(descending_result) - 1].title
        )

    def test_get_all_by_user_id_multi_field_sorting(self):
        # Arrange
        ascending_order_by_multi_field = ["+user_id", "+title"]
        descending_order_by_multi_field = ["+user_id", "-title"]
        # Act
        ascending_result = Data.get_all_by_user_id(1, ascending_order_by_multi_field)
        descending_result = Data.get_all_by_user_id(1, descending_order_by_multi_field)
        # Assert
        self.assertEqual(self.fixture.data_1.title, ascending_result.all()[0].title)
        self.assertEqual(self.fixture.data_3.title, ascending_result.all()[1].title)

        self.assertEqual(self.fixture.data_3.title, descending_result.all()[1].title)
        self.assertEqual(self.fixture.data_1.title, descending_result.all()[2].title)

    def test_data_get_all_api_without_sorting_param_use_default_data_sorting_setting(
        self,
    ):
        # Arrange
        mock_user = _create_user("1", is_superuser=True)
        # Act
        data = data_api.get_all_by_user(mock_user)
        # Assert
        self.assertListEqual(
            list(data), [self.fixture.data_1, self.fixture.data_3, self.fixture.data_5]
        )


class TestDataGetAllExceptUserId(MongoIntegrationBaseTestCase):

    fixture = access_control_data_fixture

    def test_data_get_all_except_user_id_return_collection_of_data_where_user_is_not_owner(
        self,
    ):
        # Arrange
        user_id = 1
        # Act
        result = Data.get_all_except_user_id(user_id, DATA_SORTING_FIELDS)
        # Assert
        self.assertTrue(item.user_id != user_id for item in result)

    def test_data_get_all_by_user_id_return_full_collection_of_data_from_user_does_not_exist(
        self,
    ):
        # Arrange
        user_id = 800
        # Act
        result = Data.get_all_except_user_id(user_id, DATA_SORTING_FIELDS)
        # Assert
        self.assertTrue(result.count() > 0)

    def test_get_all_except_user_id_data_ordering(self):
        # Arrange
        user_id = 1
        ascending_order_by_field = ["+title"]
        descending_order_by_field = ["-title"]
        # Act
        ascending_result = Data.get_all_except_user_id(
            user_id, ascending_order_by_field
        )
        descending_result = Data.get_all_except_user_id(
            user_id, descending_order_by_field
        )
        # Assert
        for i in range(len(ascending_result)):
            self.assertTrue(
                ascending_result.all()[i].title
                == descending_result.all()[len(ascending_result) - i - 1].title
            )

    def test_get_all_except_user_id_data_ascending_sorting(self):
        # Arrange
        ascending_order_by_field = ["+title"]
        user_id = 2
        # Act
        ascending_result = Data.get_all_except_user_id(
            user_id, ascending_order_by_field
        )
        # Assert
        self.assertTrue(self.fixture.data_1.title == ascending_result.all()[0].title)
        self.assertTrue(self.fixture.data_3.title == ascending_result.all()[1].title)

    def test_get_all_except_user_id_data_descending_sorting(self):
        # Arrange
        descending_order_by_field = ["-title"]
        user_id = 2
        # Act
        descending_result = Data.get_all_except_user_id(
            user_id, descending_order_by_field
        )
        # Assert
        self.assertTrue(
            self.fixture.data_3.title
            == descending_result.all()[len(descending_result) - 2].title
        )
        self.assertTrue(
            self.fixture.data_1.title
            == descending_result.all()[len(descending_result) - 1].title
        )

    def test_get_all_except_user_id_multi_field_sorting(self):
        # Arrange
        ascending_order_by_multi_field = ["+title", "+user_id"]
        descending_order_by_multi_field = ["+title", "-user_id"]
        # Act
        ascending_result = Data.get_all_except_user_id(
            3, ascending_order_by_multi_field
        )
        descending_result = Data.get_all_except_user_id(
            3, descending_order_by_multi_field
        )
        # Assert
        self.assertEqual(self.fixture.data_4.user_id, ascending_result.all()[4].user_id)
        self.assertEqual(self.fixture.data_5.user_id, ascending_result.all()[3].user_id)

        self.assertEqual(
            self.fixture.data_4.user_id, descending_result.all()[3].user_id
        )
        self.assertEqual(
            self.fixture.data_5.user_id, descending_result.all()[4].user_id
        )

    def test_data_get_all_except_user_api_without_sorting_param_use_default_data_sorting_setting(
        self,
    ):
        # Arrange
        mock_user = _create_user("1", is_superuser=True)
        # Act
        data = data_api.get_all_except_user(mock_user)
        # Assert
        self.assertListEqual(list(data), [self.fixture.data_2, self.fixture.data_4])


class TestExecuteQuery(MongoIntegrationTransactionTestCase):

    fixture = access_control_data_fixture

    def test_execute_query_data_ordering(self):
        # Arrange
        query = {}
        ascending_order_by_field = ["+title"]
        descending_order_by_field = ["-title"]
        # Act
        ascending_result = Data.execute_query(query, ascending_order_by_field)
        descending_result = Data.execute_query(query, descending_order_by_field)
        # Assert
        for i in range(len(ascending_result)):
            self.assertTrue(
                ascending_result.all()[i].title
                == descending_result.all()[len(ascending_result) - i - 1].title
            )

    def test_execute_query_data_ascending_sorting(self):
        # Arrange
        ascending_order_by_field = ["+title"]
        query = {}
        # Act
        ascending_result = Data.execute_query(query, ascending_order_by_field)
        # Assert
        self.assertTrue(self.fixture.data_1.title == ascending_result.all()[0].title)
        self.assertTrue(self.fixture.data_2.title == ascending_result.all()[1].title)

    def test_execute_query_data_descending_sorting(self):
        # Arrange
        descending_order_by_field = ["-title"]
        query = {}
        # Act
        descending_result = Data.execute_query(query, descending_order_by_field)
        # Assert
        self.assertTrue(
            self.fixture.data_2.title
            == descending_result.all()[len(descending_result) - 2].title
        )
        self.assertTrue(
            self.fixture.data_1.title
            == descending_result.all()[len(descending_result) - 1].title
        )

    def test_execute_query_multi_field_sorting(self):
        # Arrange
        ascending_order_by_multi_field = ["+title", "+user_id"]
        descending_order_by_multi_field = ["+title", "-user_id"]
        query = {}
        # Act
        ascending_result = Data.execute_query(query, ascending_order_by_multi_field)
        descending_result = Data.execute_query(query, descending_order_by_multi_field)
        # Assert
        self.assertEqual(self.fixture.data_4.user_id, ascending_result.all()[4].user_id)
        self.assertEqual(self.fixture.data_5.user_id, ascending_result.all()[3].user_id)

        self.assertEqual(
            self.fixture.data_4.user_id, descending_result.all()[3].user_id
        )
        self.assertEqual(
            self.fixture.data_5.user_id, descending_result.all()[4].user_id
        )

    def test_data_execute_query_api_without_sorting_param_use_default_data_sorting_setting(
        self,
    ):
        # Arrange
        mock_user = _create_user("1", is_superuser=True)
        # Act
        data = data_api.execute_query({}, mock_user)
        # Assert
        self.assertListEqual(list(data), self.fixture.data_collection)


class TestGetAllByWorkspace(MongoIntegrationBaseTestCase):

    fixture = access_control_data_fixture

    def test_get_all_by_workspace_data_ordering(self):
        # Arrange
        workspace = self.fixture.workspace_1.id
        ascending_order_by_field = ["+title"]
        descending_order_by_field = ["-title"]
        # Act
        ascending_result = Data.get_all_by_workspace(
            workspace, ascending_order_by_field
        )
        descending_result = Data.get_all_by_workspace(
            workspace, descending_order_by_field
        )
        # Assert
        for i in range(len(ascending_result)):
            self.assertTrue(
                ascending_result.all()[i].title
                == descending_result.all()[len(ascending_result) - i - 1].title
            )

    def test_get_all_by_workspace_data_ascending_sorting(self):
        # Arrange
        ascending_order_by_field = ["+title"]
        workspace = self.fixture.workspace_1.id
        # Act
        ascending_result = Data.get_all_by_workspace(
            workspace, ascending_order_by_field
        )
        # Assert
        self.assertTrue(self.fixture.data_3.title == ascending_result.all()[0].title)
        self.assertTrue(self.fixture.data_5.title == ascending_result.all()[1].title)

    def test_get_all_by_workspace_data_descending_sorting(self):
        # Arrange
        descending_order_by_field = ["-title"]
        workspace = self.fixture.workspace_1.id
        # Act
        descending_result = Data.get_all_by_workspace(
            workspace, descending_order_by_field
        )
        # Assert
        self.assertTrue(
            self.fixture.data_5.title
            == descending_result.all()[len(descending_result) - 2].title
        )
        self.assertTrue(
            self.fixture.data_3.title
            == descending_result.all()[len(descending_result) - 1].title
        )

    def test_get_all_by_workspace_multi_field_sorting(self):
        # Arrange
        ascending_order_by_multi_field = ["+workspace", "+title"]
        descending_order_by_multi_field = ["+workspace", "-title"]
        workspace = self.fixture.workspace_1.id
        # Act
        ascending_result = Data.get_all_by_workspace(
            workspace, ascending_order_by_multi_field
        )
        descending_result = Data.get_all_by_workspace(
            workspace, descending_order_by_multi_field
        )
        # Assert
        self.assertEqual(self.fixture.data_3.user_id, ascending_result.all()[0].user_id)
        self.assertEqual(self.fixture.data_5.user_id, ascending_result.all()[1].user_id)

        self.assertEqual(
            self.fixture.data_3.user_id, descending_result.all()[1].user_id
        )
        self.assertEqual(
            self.fixture.data_5.user_id, descending_result.all()[0].user_id
        )

    def test_data_get_all_by_workspace_api_without_sorting_param_use_default_data_sorting_setting(
        self,
    ):
        # Arrange
        mock_user = _create_user("1", is_superuser=True)
        workspace = self.fixture.workspace_1.id
        # Act
        data = data_api.get_all_by_workspace(workspace, mock_user)
        # Assert
        self.assertListEqual(list(data), [self.fixture.data_3, self.fixture.data_5])


class TestGetAllByListTemplate(MongoIntegrationBaseTestCase):
    fixture = access_control_data_fixture

    def test_returns_data_object(self):
        result = Data.get_all_by_list_workspace(
            [self.fixture.workspace_1.id, self.fixture.workspace_2.id],
            DATA_SORTING_FIELDS,
        )
        self.assertTrue(all(isinstance(item, Data) for item in result))

    def test_returns_correct_count(self):
        result = Data.get_all_by_list_workspace(
            [self.fixture.workspace_1.id, self.fixture.workspace_2.id],
            DATA_SORTING_FIELDS,
        )
        self.assertEqual(len(result), 3)

    def test_none_returns_data_object(self):
        result = Data.get_all_by_list_workspace([None], DATA_SORTING_FIELDS)
        self.assertTrue(all(isinstance(item, Data) for item in result))

    def test_none_returns_correct_count(self):
        result = Data.get_all_by_list_workspace([None], DATA_SORTING_FIELDS)
        self.assertEqual(len(result), 2)

    def test_empty_list_returns_no_data(self):
        result = Data.get_all_by_list_workspace([], DATA_SORTING_FIELDS)
        self.assertEqual(len(result), 0)

    def test_invalid_workspace_returns_no_data(self):
        result = Data.get_all_by_list_workspace([ObjectId()], DATA_SORTING_FIELDS)
        self.assertEqual(len(result), 0)


class TestGetAllByTemplatesAndWorkspaces(MongoIntegrationBaseTestCase):
    fixture = access_control_data_fixture

    def test_returns_data_object(self):
        result = Data.get_all_by_templates_and_workspaces(
            [self.fixture.template.id],
            [self.fixture.workspace_1.id, self.fixture.workspace_2.id],
            DATA_SORTING_FIELDS,
        )
        self.assertTrue(all(isinstance(item, Data) for item in result))

    def test_returns_correct_count(self):
        result = Data.get_all_by_templates_and_workspaces(
            [self.fixture.template.id],
            [self.fixture.workspace_1.id, self.fixture.workspace_2.id],
            DATA_SORTING_FIELDS,
        )
        self.assertEqual(len(result), 3)

    def test_null_workspace_returns_data_object(self):
        result = Data.get_all_by_templates_and_workspaces(
            [self.fixture.template.id], [None], DATA_SORTING_FIELDS
        )
        self.assertTrue(all(isinstance(item, Data) for item in result))

    def test_null_workspace_returns_correct_count(self):
        result = Data.get_all_by_templates_and_workspaces(
            [self.fixture.template.id], [None], DATA_SORTING_FIELDS
        )
        self.assertEqual(len(result), 2)

    def test_empty_workspaces_returns_no_data(self):
        result = Data.get_all_by_templates_and_workspaces(
            [self.fixture.template.id], [], DATA_SORTING_FIELDS
        )
        self.assertEqual(len(result), 0)

    def test_empty_templates_returns_no_data(self):
        result = Data.get_all_by_templates_and_workspaces(
            [], [None], DATA_SORTING_FIELDS
        )
        self.assertEqual(len(result), 0)

    def test_invalid_workspace_returns_no_data(self):
        result = Data.get_all_by_templates_and_workspaces(
            [self.fixture.template.id], [ObjectId()], DATA_SORTING_FIELDS
        )
        self.assertEqual(len(result), 0)

    def test_invalid_template_returns_no_data(self):
        result = Data.get_all_by_templates_and_workspaces(
            [ObjectId()], [None], DATA_SORTING_FIELDS
        )
        self.assertEqual(len(result), 0)


class TestGetAllByUserAndWorkspace(MongoIntegrationBaseTestCase):

    fixture = access_control_data_fixture

    def test_get_all_data_from_user_and_from_workspace_for_user_within_workspace(self):
        # Arrange
        mock_user = create_mock_user("1", is_superuser=False)
        # Act
        data = Data.get_all_by_user_and_workspace(
            mock_user.id,
            [self.fixture.workspace_1, self.fixture.workspace_2],
            ["+title"],
        )
        # Assert
        self.assertListEqual(
            list(data),
            [
                self.fixture.data_1,
                self.fixture.data_3,
                self.fixture.data_4,
                self.fixture.data_5,
            ],
        )

    def test_get_all_data_from_user_and_from_workspace_data_ordering(self):
        # Arrange
        mock_user = create_mock_user("1", is_superuser=False)
        ascending_order_by_field = ["+title"]
        descending_order_by_field = ["-title"]
        # Act
        ascending_result = Data.get_all_by_user_and_workspace(
            mock_user.id, [self.fixture.workspace_1], ascending_order_by_field
        )
        descending_result = Data.get_all_by_user_and_workspace(
            mock_user.id, [self.fixture.workspace_1], descending_order_by_field
        )
        # Assert
        for i in range(len(ascending_result)):
            self.assertTrue(
                ascending_result.all()[i].title
                == descending_result.all()[len(ascending_result) - i - 1].title
            )

    def test_get_all_data_from_user_and_from_workspace_data_ascending_sorting(self):
        # Arrange
        ascending_order_by_field = ["+title"]
        mock_user = create_mock_user("1", is_superuser=False)
        # Act
        ascending_result = Data.get_all_by_user_and_workspace(
            mock_user.id, [self.fixture.workspace_1], ascending_order_by_field
        )
        # Assert
        self.assertTrue(self.fixture.data_1.title == ascending_result.all()[0].title)
        self.assertTrue(self.fixture.data_3.title == ascending_result.all()[1].title)

    def test_get_all_data_from_user_and_from_workspace_data_descending_sorting(self):
        # Arrange
        descending_order_by_field = ["-title"]
        mock_user = create_mock_user("1", is_superuser=False)
        # Act
        descending_result = Data.get_all_by_user_and_workspace(
            mock_user.id, [self.fixture.workspace_1], descending_order_by_field
        )
        # Assert
        self.assertTrue(
            self.fixture.data_3.title
            == descending_result.all()[len(descending_result) - 2].title
        )
        self.assertTrue(
            self.fixture.data_1.title
            == descending_result.all()[len(descending_result) - 1].title
        )

    def test_get_all_data_from_user_and_from_workspace_multi_field_sorting(self):
        # Arrange
        ascending_order_by_multi_field = ["+template", "+title"]
        descending_order_by_multi_field = ["+template", "-title"]
        mock_user = create_mock_user("1", is_superuser=False)
        # Act
        ascending_result = Data.get_all_by_user_and_workspace(
            mock_user.id, [self.fixture.workspace_1], ascending_order_by_multi_field
        )
        descending_result = Data.get_all_by_user_and_workspace(
            mock_user.id, [self.fixture.workspace_1], descending_order_by_multi_field
        )
        # Assert
        self.assertEqual(self.fixture.data_1.title, ascending_result.all()[0].title)
        self.assertEqual(self.fixture.data_3.user_id, ascending_result.all()[1].user_id)

        self.assertEqual(
            self.fixture.data_3.user_id, descending_result.all()[1].user_id
        )
        self.assertEqual(
            self.fixture.data_1.user_id, descending_result.all()[2].user_id
        )


class TestDataMigration(MongoIntegrationTransactionTestCase):
    fixture = fixture_data_template

    def setUp(self):
        """Insert needed data.

        Returns:

        """
        super().setUp()
        self.fixture.insert_data()

    @patch.object(data_api, "get_by_id")
    @patch.object(template_api, "get")
    def test_data_template_validation_success_for_one_data(
        self, template_get, data_get_by_id
    ):
        # Arrange
        request_user = UserFixtures().create_super_user("admin_test")
        template_get.return_value = self.fixture.template_2
        data_get_by_id.return_value = self.fixture.data_1

        # Act
        response = data_task.async_migration_task(
            [self.fixture.data_1.id], self.fixture.template_2.id, request_user.id, False
        )

        # Assert
        expected_result = {"valid": [str(self.fixture.data_1.id)], "wrong": []}
        self.assertEqual(response, expected_result)

    @patch.object(data_api, "get_by_id")
    @patch.object(system_api, "get_template_by_id")
    def test_data_template_validation_success_for_multi_data(
        self, template_get, data_get_by_id
    ):
        # Arrange
        data_get_by_id.side_effect = [self.fixture.data_1, self.fixture.data_2]
        template_get.return_value = self.fixture.template_2
        request_user = UserFixtures().create_super_user("admin_test")

        # Act
        response = data_task.async_migration_task(
            [self.fixture.data_1.id, self.fixture.data_2.id],
            self.fixture.template_2.id,
            request_user.id,
            False,
        )

        # Assert
        expected_result = {
            "valid": [str(self.fixture.data_1.id), str(self.fixture.data_2.id)],
            "wrong": [],
        }
        self.assertEqual(response, expected_result)

    @patch.object(data_api, "get_by_id")
    @patch.object(template_api, "get")
    def test_data_template_validation_error_for_one_data(
        self, template_get, data_get_by_id
    ):
        # Arrange
        data_get_by_id.return_value = self.fixture.data_5
        template_get.return_value = self.fixture.template_2
        request_user = UserFixtures().create_super_user("admin_test")

        # Act
        response = data_task.async_migration_task(
            [self.fixture.data_5.id],
            self.fixture.template_2.id,
            request_user.id,
            False,
        )

        # Assert
        expected_result = {"valid": [], "wrong": [str(self.fixture.data_5.id)]}
        self.assertEqual(response, expected_result)

    @patch.object(data_api, "get_by_id")
    @patch.object(template_api, "get")
    def test_data_template_validation_for_multi_data(
        self, template_get, data_get_by_id
    ):
        # Arrange
        data_get_by_id.side_effect = [self.fixture.data_1, self.fixture.data_5]
        template_get.return_value = self.fixture.template_2
        request_user = UserFixtures().create_super_user("admin_test")

        # Act
        response = data_task.async_migration_task(
            [self.fixture.data_1.id, self.fixture.data_5.id],
            self.fixture.template_2.id,
            request_user.id,
            False,
        )

        # Assert
        expected_result = {
            "valid": [str(self.fixture.data_1.id)],
            "wrong": [str(self.fixture.data_5.id)],
        }
        self.assertEqual(response, expected_result)

    @patch.object(data_api, "execute_query")
    @patch.object(data_api, "get_by_id")
    @patch.object(template_api, "get")
    def test_data_template_group_validation_success(
        self, template_get, data_get_by_id, data_execute_query
    ):
        # Arrange
        mock_query_set = {
            "values_list": lambda param: [
                self.fixture.data_1.id,
                self.fixture.data_2.id,
            ],
            "count": lambda: 2,
        }
        data_execute_query.return_value = SimpleNamespace(**mock_query_set)
        data_get_by_id.side_effect = [
            self.fixture.data_1,
            self.fixture.data_2,
        ]

        template_get.return_value = self.fixture.template_2
        request_user = UserFixtures().create_super_user("admin_test")

        # Act
        response = data_task.async_template_migration_task(
            [self.fixture.template_1.id],
            self.fixture.template_2.id,
            request_user.id,
            False,
        )

        # Assert
        expected_result = {
            "valid": [str(self.fixture.data_1.id), str(self.fixture.data_2.id)],
            "wrong": [],
        }
        self.assertEqual(response, expected_result)

    @patch.object(data_api, "execute_query")
    @patch.object(data_api, "get_by_id")
    @patch.object(template_api, "get")
    def test_data_template_group_validation_error(
        self, template_get, data_get_by_id, data_execute_query
    ):
        # Arrange
        mock_query_set = {
            "values_list": lambda param: [
                self.fixture.data_4.id,
                self.fixture.data_5.id,
            ],
            "count": lambda: 2,
        }
        data_execute_query.return_value = SimpleNamespace(**mock_query_set)
        data_get_by_id.side_effect = [
            self.fixture.data_4,
            self.fixture.data_5,
        ]

        template_get.return_value = self.fixture.template_1
        request_user = UserFixtures().create_super_user("admin_test")

        # Act
        response = data_task.async_template_migration_task(
            [self.fixture.template_3.id],
            self.fixture.template_1.id,
            request_user.id,
            False,
        )

        # Assert
        expected_result = {
            "valid": [],
            "wrong": [str(self.fixture.data_4.id), str(self.fixture.data_5.id)],
        }
        self.assertEqual(response, expected_result)

    @patch.object(data_api, "get_by_id")
    @patch.object(data_api, "upsert")
    @patch.object(template_api, "get")
    def test_data_template_migration_success_for_one_data(
        self, template_get, data_upsert, data_get_by_id
    ):
        # Arrange
        request_user = UserFixtures().create_super_user("admin_test")
        data_upsert.side_effect = mock_upsert
        template_get.return_value = self.fixture.template_2
        data_get_by_id.return_value = self.fixture.data_1

        # Act
        response = data_task.async_migration_task(
            [self.fixture.data_1.id], self.fixture.template_2.id, request_user.id, True
        )

        # Assert
        migrated_data = data_api.get_by_id(self.fixture.data_1.id, request_user)
        self.assertEqual(migrated_data.template.id, self.fixture.template_2.id)

    @patch.object(data_api, "get_by_id")
    @patch.object(data_api, "upsert")
    @patch.object(template_api, "get")
    def test_data_template_migration_success_for_multi_data(
        self, template_get, data_upsert, data_get_by_id
    ):
        # Arrange
        data_get_by_id.side_effect = [self.fixture.data_1, self.fixture.data_2]
        template_get.return_value = self.fixture.template_2
        data_upsert.side_effect = mock_upsert
        request_user = UserFixtures().create_super_user("admin_test")

        # Act
        response = data_task.async_migration_task(
            [self.fixture.data_1.id, self.fixture.data_2.id],
            self.fixture.template_2.id,
            request_user.id,
            False,
        )

        # Assert
        migrated_data_template = [
            self.fixture.data_1.template.id,
            self.fixture.data_2.template.id,
        ]
        self.assertListEqual(
            migrated_data_template,
            [self.fixture.template_2.id, self.fixture.template_2.id],
        )

    @patch.object(data_api, "get_by_id")
    @patch.object(data_api, "upsert")
    @patch.object(template_api, "get")
    def test_data_template_migration_error_for_one_data(
        self, template_get, data_upsert, data_get_by_id
    ):
        # Arrange
        request_user = UserFixtures().create_super_user("admin_test")
        data_upsert.side_effect = mock_upsert
        template_get.return_value = self.fixture.template_2
        data_get_by_id.return_value = self.fixture.data_5

        # Act
        response = data_task.async_migration_task(
            [self.fixture.data_5.id], self.fixture.template_2.id, request_user.id, True
        )

        # Assert
        migrated_data = data_api.get_by_id(self.fixture.data_5.id, request_user)
        self.assertEqual(migrated_data.template.id, self.fixture.template_2.id)

    @patch.object(data_api, "get_by_id")
    @patch.object(data_api, "upsert")
    @patch.object(template_api, "get")
    def test_data_template_migration_for_multi_data(
        self, template_get, data_upsert, data_get_by_id
    ):
        # Arrange
        data_get_by_id.side_effect = [self.fixture.data_1, self.fixture.data_5]
        template_get.return_value = self.fixture.template_2
        data_upsert.side_effect = mock_upsert
        request_user = UserFixtures().create_super_user("admin_test")

        # Act
        response = data_task.async_migration_task(
            [self.fixture.data_1.id, self.fixture.data_5.id],
            self.fixture.template_2.id,
            request_user.id,
            False,
        )

        # Assert
        migrated_data_template = [
            self.fixture.data_1.template.id,
            self.fixture.data_5.template.id,
        ]
        self.assertListEqual(
            migrated_data_template,
            [self.fixture.template_2.id, self.fixture.template_2.id],
        )

        @patch.object(data_api, "execute_query")
        @patch.object(template_api, "get")
        def test_data_template_group_migration_success(
            self, template_get, data_execute_query
        ):
            # Arrange
            data_execute_query.return_value = [self.fixture.data_1, self.fixture.data_2]
            template_get.return_value = self.fixture.template_2
            request_user = UserFixtures().create_super_user("admin_test")

            # Act
            response = data_task.async_template_migration_task(
                [self.fixture.template_1.id],
                self.fixture.template_2.id,
                request_user.id,
                True,
            )

            # Assert
            migrated_data_template = [
                self.fixture.data_1.template.id,
                self.fixture.data_2.template.id,
            ]
            self.assertListEqual(
                migrated_data_template,
                [self.fixture.template_2.id, self.fixture.template_2.id],
            )

    @patch.object(data_api, "execute_query")
    @patch.object(data_api, "get_by_id")
    @patch.object(system_api, "get_template_by_id")
    def test_data_template_group_migration_error(
        self, template_get, data_get_by_id, data_execute_query
    ):
        # Arrange
        mock_query_set = {
            "values_list": lambda param: [
                self.fixture.data_4.id,
                self.fixture.data_5.id,
            ],
            "count": lambda: 2,
        }
        data_execute_query.return_value = SimpleNamespace(**mock_query_set)
        data_get_by_id.side_effect = [
            self.fixture.data_4,
            self.fixture.data_5,
        ]

        template_get.return_value = self.fixture.template_2
        request_user = UserFixtures().create_super_user("admin_test")

        # Act
        response = data_task.async_template_migration_task(
            [self.fixture.template_3.id],
            self.fixture.template_1.id,
            request_user.id,
            True,
        )

        # Assert
        migrated_data_template = [
            self.fixture.data_4.template.id,
            self.fixture.data_5.template.id,
        ]
        self.assertListEqual(
            migrated_data_template,
            [self.fixture.template_3.id, self.fixture.template_3.id],
        )


@patch.object(data_api, "get_by_id")
@patch.object(data_api, "upsert")
@patch.object(template_api, "get")
def test_result_data_template_migration_success_for_one_data(
    self, template_get, data_upsert, data_get_by_id
):
    # Arrange
    request_user = UserFixtures().create_super_user("admin_test")
    data_upsert.side_effect = mock_upsert
    template_get.return_value = self.fixture.template_2
    data_get_by_id.return_value = self.fixture.data_1

    # Act
    response = data_task.async_migration_task(
        [self.fixture.data_1.id], self.fixture.template_2.id, request_user.id, True
    )

    # Assert
    expected_result = {"valid": [str(self.fixture.data_1.id)], "wrong": []}
    self.assertEqual(response, expected_result)


@patch.object(data_api, "get_by_id")
@patch.object(data_api, "upsert")
@patch.object(template_api, "get")
def test_result_data_template_migration_success_for_multi_data(
    self, template_get, data_upsert, data_get_by_id
):
    # Arrange
    data_get_by_id.side_effect = [self.fixture.data_1, self.fixture.data_2]
    template_get.return_value = self.fixture.template_2
    data_upsert.side_effect = mock_upsert
    request_user = UserFixtures().create_super_user("admin_test")

    # Act
    response = data_task.async_migration_task(
        [self.fixture.data_1.id, self.fixture.data_2.id],
        self.fixture.template_2.id,
        request_user.id,
        False,
    )

    # Assert
    expected_result = {
        "valid": [str(self.fixture.data_1.id), str(self.fixture.data_2.id)],
        "wrong": [],
    }
    self.assertEqual(response, expected_result)


@patch.object(data_api, "get_by_id")
@patch.object(data_api, "upsert")
@patch.object(template_api, "get")
def test_result_data_template_migration_error_for_one_data(
    self, template_get, data_upsert, data_get_by_id
):
    # Arrange
    request_user = UserFixtures().create_super_user("admin_test")
    data_upsert.side_effect = mock_upsert
    template_get.return_value = self.fixture.template_2
    data_get_by_id.return_value = self.fixture.data_5

    # Act
    response = data_task.async_migration_task(
        [self.fixture.data_5.id], self.fixture.template_2.id, request_user.id, True
    )

    # Assert
    expected_result = {"valid": [], "wrong": [str(self.fixture.data_5.id)]}
    self.assertEqual(response, expected_result)


@patch.object(data_api, "get_by_id")
@patch.object(data_api, "upsert")
@patch.object(template_api, "get")
def test_result_data_template_migration_for_multi_data(
    self, template_get, data_upsert, data_get_by_id
):
    # Arrange
    data_get_by_id.side_effect = [self.fixture.data_1, self.fixture.data_5]
    template_get.return_value = self.fixture.template_2
    data_upsert.side_effect = mock_upsert
    request_user = UserFixtures().create_super_user("admin_test")

    # Act
    response = data_task.async_migration_task(
        [self.fixture.data_1.id, self.fixture.data_5.id],
        self.fixture.template_2.id,
        request_user.id,
        False,
    )

    # Assert
    expected_result = {
        "valid": [str(self.fixture.data_1.id)],
        "wrong": [str(self.fixture.data_5.id)],
    }
    self.assertEqual(response, expected_result)

    @patch.object(data_api, "execute_query")
    @patch.object(template_api, "get")
    def test_result_data_template_group_migration_success(
        self, template_get, data_execute_query
    ):
        # Arrange
        data_execute_query.return_value = [self.fixture.data_1, self.fixture.data_2]
        template_get.return_value = self.fixture.template_2
        request_user = UserFixtures().create_super_user("admin_test")

        # Act
        response = data_task.async_template_migration_task(
            [self.fixture.template_1.id],
            self.fixture.template_2.id,
            request_user.id,
            True,
        )

        # Assert
        expected_result = {
            "valid": [str(self.fixture.data_1.id), str(self.fixture.data_2.id)],
            "wrong": [],
        }
        self.assertEqual(response, expected_result)


@patch.object(data_api, "execute_query")
@patch.object(template_api, "get")
def test_result_data_template_group_migration_error(
    self, template_get, data_execute_query
):
    # Arrange
    data_execute_query.return_value = [self.fixture.data_4, self.fixture.data_5]
    template_get.return_value = self.fixture.template_2
    request_user = UserFixtures().create_super_user("admin_test")

    # Act
    response = data_task.async_template_migration_task(
        [self.fixture.template_3.id],
        self.fixture.template_1.id,
        request_user.id,
        True,
    )

    # Assert
    expected_result = {
        "valid": [],
        "wrong": [str(self.fixture.data_4.id), str(self.fixture.data_5.id)],
    }
    self.assertEqual(response, expected_result)


def mock_upsert(data, user):
    if data.xml_content is None:
        raise exceptions.ApiError("Unable to save data: xml_content field is not set.")

    data.last_modification_date = datetime.datetime.now(pytz.utc)
    check_xml_file_is_valid(data)
    return data.save()


def _create_user(user_id, is_superuser=False):
    return create_mock_user(user_id, is_superuser=is_superuser)



"""BLOB component utils
"""
from django.urls import reverse

from core_main_app.settings import SERVER_URI


def get_blob_download_uri(blob, request):
    """Return download url for the blob.

    Args:
        blob:
        request:

    Returns:

    """
    # get URI to download blob
    blob_download_uri = SERVER_URI + reverse(
        "core_main_app_rest_blob_download", kwargs={"pk": str(blob.id)}
    )

    return blob_download_uri



""" Authentication tests for Data REST API
"""
from django.test import SimpleTestCase
from mock import Mock
from mock.mock import patch
from rest_framework import status
from rest_framework.response import Response
from rest_framework.status import HTTP_200_OK

from core_main_app.components.data import api as data_api
from core_main_app.components.data.models import Data
from core_main_app.components.workspace import api as workspace_api
from core_main_app.rest.data import views as data_rest_views
from core_main_app.rest.data.abstract_views import AbstractExecuteLocalQueryView
from core_main_app.rest.data.admin_serializers import AdminDataSerializer
from core_main_app.rest.data.serializers import (
    DataSerializer,
    DataWithTemplateInfoSerializer,
)
from core_main_app.rest.data.views import Validation as data_validation
from core_main_app.rest.data.views import Migration as data_migration
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.tests_tools.RequestMock import RequestMock


class TestDataListPostPermissions(SimpleTestCase):
    def test_anonymous_returns_http_403(self):
        response = RequestMock.do_request_post(data_rest_views.DataList.as_view(), None)

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(DataSerializer, "is_valid")
    @patch.object(DataSerializer, "save")
    @patch.object(DataSerializer, "data")
    def test_authenticated_returns_http_201(
        self, data_serializer_data, data_serializer_save, data_serializer_valid
    ):
        data_serializer_valid.return_value = True
        data_serializer_save.return_value = None
        data_serializer_data.return_value = {}

        mock_user = create_mock_user("1")

        response = RequestMock.do_request_post(
            data_rest_views.DataList.as_view(), mock_user
        )

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

    @patch.object(DataSerializer, "is_valid")
    @patch.object(DataSerializer, "save")
    @patch.object(DataSerializer, "data")
    def test_staff_returns_http_201(
        self, data_serializer_data, data_serializer_save, data_serializer_valid
    ):
        data_serializer_valid.return_value = True
        data_serializer_save.return_value = None
        data_serializer_data.return_value = {}

        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_post(
            data_rest_views.DataList.as_view(), mock_user
        )

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)


class TestDataListGetPermissions(SimpleTestCase):
    def test_anonymous_returns_http_403(self):
        response = RequestMock.do_request_get(data_rest_views.DataList.as_view(), None)

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(Data, "get_all_by_user_id")
    def test_authenticated_returns_http_200(self, data_get_all_by_user):
        data_get_all_by_user.return_value = {}

        mock_user = create_mock_user("1")

        response = RequestMock.do_request_get(
            data_rest_views.DataList.as_view(), mock_user
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(Data, "get_all_by_user_id")
    def test_staff_returns_http_200(self, data_get_all_by_user):
        data_get_all_by_user.return_value = {}

        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_get(
            data_rest_views.DataList.as_view(), mock_user
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestAdminDataListPostPermissions(SimpleTestCase):
    def test_anonymous_returns_http_403(self):
        response = RequestMock.do_request_post(
            data_rest_views.AdminDataList.as_view(), None
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_authenticated_returns_http_403(self):
        mock_user = create_mock_user("1")
        response = RequestMock.do_request_post(
            data_rest_views.AdminDataList.as_view(), mock_user
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_staff_returns_http_403(self):
        mock_user = create_mock_user("1", is_staff=True)
        response = RequestMock.do_request_post(
            data_rest_views.AdminDataList.as_view(), mock_user
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(AdminDataSerializer, "is_valid")
    @patch.object(AdminDataSerializer, "save")
    @patch.object(AdminDataSerializer, "data")
    def test_superuser_returns_http_201(
        self, data_serializer_data, data_serializer_save, data_serializer_valid
    ):
        data_serializer_valid.return_value = True
        data_serializer_save.return_value = None
        data_serializer_data.return_value = {}

        mock_user = create_mock_user("1", is_staff=True, is_superuser=True)

        response = RequestMock.do_request_post(
            data_rest_views.AdminDataList.as_view(), mock_user
        )

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)


class TestAdminDataListGetPermissions(SimpleTestCase):
    def test_anonymous_returns_http_403(self):
        response = RequestMock.do_request_get(
            data_rest_views.AdminDataList.as_view(), None
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_authenticated_returns_http_403(self):
        mock_user = create_mock_user("1")
        response = RequestMock.do_request_get(
            data_rest_views.AdminDataList.as_view(), mock_user
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_staff_returns_http_403(self):
        mock_user = create_mock_user("1", is_staff=True)
        response = RequestMock.do_request_get(
            data_rest_views.AdminDataList.as_view(), mock_user
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(Data, "get_all_by_user_id")
    def test_superuser_returns_http_200(self, data_get_all_by_user):
        data_get_all_by_user.return_value = {}

        mock_user = create_mock_user("1", is_staff=True, is_superuser=True)

        response = RequestMock.do_request_get(
            data_rest_views.AdminDataList.as_view(), mock_user
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestDataListByWorkspaceGetPermissions(SimpleTestCase):
    def test_anonymous_returns_http_403(self):
        response = RequestMock.do_request_get(
            data_rest_views.DataListByWorkspace.as_view(),
            None,
            param={"workspace_id": 0},
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(data_api, "get_all_by_workspace")
    def test_authenticated_returns_http_200(self, data_get_all_by_workspace):
        data_get_all_by_workspace.return_value = {}

        mock_user = create_mock_user("1")

        response = RequestMock.do_request_get(
            data_rest_views.DataListByWorkspace.as_view(),
            mock_user,
            param={"workspace_id": 0},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(data_api, "get_all_by_workspace")
    def test_staff_returns_http_200(self, data_get_all_by_workspace):
        data_get_all_by_workspace.return_value = {}

        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_get(
            data_rest_views.DataListByWorkspace.as_view(),
            mock_user,
            param={"workspace_id": 0},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestDataDetailGetPermissions(SimpleTestCase):
    @patch.object(DataSerializer, "data")
    @patch.object(data_api, "get_by_id")
    def test_anonymous_returns_http_200(
        self, mock_data_api_get_by_id, mock_data_serializer_data
    ):
        mock_data_api_get_by_id.return_value = None
        mock_data_serializer_data.return_value = []

        response = RequestMock.do_request_get(
            data_rest_views.DataDetail.as_view(), None, param={"pk": 0}
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(DataSerializer, "data")
    @patch.object(data_api, "get_by_id")
    def test_authenticated_returns_http_200(
        self, mock_data_api_get_by_id, mock_data_serializer_data
    ):
        mock_data_api_get_by_id.return_value = None
        mock_data_serializer_data.return_value = []

        mock_user = create_mock_user("1")
        response = RequestMock.do_request_get(
            data_rest_views.DataDetail.as_view(), mock_user, param={"pk": 0}
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(DataSerializer, "data")
    @patch.object(data_api, "get_by_id")
    def test_staff_returns_http_200(
        self, mock_data_api_get_by_id, mock_data_serializer_data
    ):
        mock_data_api_get_by_id.return_value = None
        mock_data_serializer_data.return_value = []

        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_get(
            data_rest_views.DataDetail.as_view(), mock_user, param={"pk": 0}
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestDataDetailDeletePermissions(SimpleTestCase):
    def test_anonymous_returns_http_403(self):
        response = RequestMock.do_request_delete(
            data_rest_views.DataDetail.as_view(), None, param={"pk": 0}
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(data_api, "delete")
    @patch.object(data_api, "get_by_id")
    def test_authenticated_returns_http_204(
        self, mock_data_api_get_by_id, mock_data_api_delete
    ):
        mock_data_api_get_by_id.return_value = None
        mock_data_api_delete.return_value = None

        mock_user = create_mock_user("1")

        response = RequestMock.do_request_delete(
            data_rest_views.DataDetail.as_view(), mock_user, param={"pk": 0}
        )

        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)

    @patch.object(data_api, "delete")
    @patch.object(data_api, "get_by_id")
    def test_staff_returns_http_204(
        self, mock_data_api_get_by_id, mock_data_api_delete
    ):
        mock_data_api_get_by_id.return_value = None
        mock_data_api_delete.return_value = None

        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_delete(
            data_rest_views.DataDetail.as_view(), mock_user, param={"pk": 0}
        )

        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)


class TestDataDetailPatchPermissions(SimpleTestCase):
    def test_anonymous_returns_http_403(self):
        response = RequestMock.do_request_patch(
            data_rest_views.DataDetail.as_view(), None, param={"pk": 0}
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(DataSerializer, "data")
    @patch.object(DataSerializer, "save")
    @patch.object(DataSerializer, "is_valid")
    @patch.object(data_api, "get_by_id")
    def test_authenticated_returns_http_200(
        self,
        mock_data_api_get_by_id,
        mock_data_serializer_is_valid,
        mock_data_serializer_save,
        mock_data_serializer_data,
    ):
        mock_data_api_get_by_id.return_value = []
        mock_data_serializer_is_valid.return_value = True
        mock_data_serializer_save.return_valie = None
        mock_data_serializer_data.return_value = []

        mock_user = create_mock_user("1")

        response = RequestMock.do_request_patch(
            data_rest_views.DataDetail.as_view(), mock_user, param={"pk": 0}
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(DataSerializer, "data")
    @patch.object(DataSerializer, "save")
    @patch.object(DataSerializer, "is_valid")
    @patch.object(data_api, "get_by_id")
    def test_staff_returns_http_200(
        self,
        mock_data_api_get_by_id,
        mock_data_serializer_is_valid,
        mock_data_serializer_save,
        mock_data_serializer_data,
    ):
        mock_data_api_get_by_id.return_value = []
        mock_data_serializer_is_valid.return_value = True
        mock_data_serializer_save.return_valie = None
        mock_data_serializer_data.return_value = []

        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_patch(
            data_rest_views.DataDetail.as_view(), mock_user, param={"pk": 0}
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestDataDownloadGetPermissions(SimpleTestCase):
    @patch.object(data_api, "get_by_id")
    def test_anonymous_returns_http_200(self, mock_data_api_get_by_id):
        data_object = Mock()
        data_object.xml_content = "<tag attr='attr_val'>tag_val</tag>"
        data_object.title = "mock.xml"

        mock_data_api_get_by_id.return_value = data_object

        response = RequestMock.do_request_get(
            data_rest_views.DataDownload.as_view(), None, param={"pk": 0}
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(data_api, "get_by_id")
    def test_authenticated_returns_http_200(self, mock_data_api_get_by_id):
        data_object = Mock()
        data_object.xml_content = "<tag attr='attr_val'>tag_val</tag>"
        data_object.title = "mock.xml"

        mock_data_api_get_by_id.return_value = data_object

        mock_user = create_mock_user("1")

        response = RequestMock.do_request_get(
            data_rest_views.DataDownload.as_view(), mock_user, param={"pk": 0}
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(data_api, "get_by_id")
    def test_staff_returns_http_200(self, mock_data_api_get_by_id):
        data_object = Mock()
        data_object.xml_content = "<tag attr='attr_val'>tag_val</tag>"
        data_object.title = "mock.xml"

        mock_data_api_get_by_id.return_value = data_object

        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_get(
            data_rest_views.DataDownload.as_view(), mock_user, param={"pk": 0}
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestDataGetFullGetPermissions(SimpleTestCase):
    @patch.object(DataWithTemplateInfoSerializer, "data")
    @patch.object(data_api, "get_by_id")
    def test_anonymous_returns_http_200(
        self, mock_data_api_get_by_id, mock_data_with_template_serializer_data
    ):
        mock_data_api_get_by_id.return_value = None
        mock_data_with_template_serializer_data.return_value = []

        response = RequestMock.do_request_get(
            data_rest_views.get_by_id_with_template_info, None, data={"id": 0}
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(DataWithTemplateInfoSerializer, "data")
    @patch.object(data_api, "get_by_id")
    def test_authenticated_returns_http_200(
        self, mock_data_api_get_by_id, mock_data_with_template_serializer_data
    ):
        mock_data_api_get_by_id.return_value = None
        mock_data_with_template_serializer_data.return_value = []

        mock_user = create_mock_user("1")

        response = RequestMock.do_request_get(
            data_rest_views.get_by_id_with_template_info, mock_user, data={"id": 0}
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(DataWithTemplateInfoSerializer, "data")
    @patch.object(data_api, "get_by_id")
    def test_staff_returns_http_200(
        self, mock_data_api_get_by_id, mock_data_with_template_serializer_data
    ):
        mock_data_api_get_by_id.return_value = None
        mock_data_with_template_serializer_data.return_value = []

        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_get(
            data_rest_views.get_by_id_with_template_info, mock_user, data={"id": 0}
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestDataLocalQueryGetPermissions(SimpleTestCase):
    @patch.object(AbstractExecuteLocalQueryView, "execute_query")
    def test_anonymous_returns_http_200(self, mock_abstract_local_query_execute_query):
        mock_abstract_local_query_execute_query.return_value = Response(
            status=HTTP_200_OK
        )

        response = RequestMock.do_request_get(
            data_rest_views.ExecuteLocalQueryView.as_view(), None, data={"query": "{}"}
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(AbstractExecuteLocalQueryView, "execute_query")
    def test_authenticated_returns_http_200(
        self, mock_abstract_local_query_execute_query
    ):
        mock_abstract_local_query_execute_query.return_value = Response(
            status=HTTP_200_OK
        )

        mock_user = create_mock_user("1")

        response = RequestMock.do_request_get(
            data_rest_views.ExecuteLocalQueryView.as_view(),
            mock_user,
            data={"query": "{}"},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(AbstractExecuteLocalQueryView, "execute_query")
    def test_staff_returns_http_200(self, mock_abstract_local_query_execute_query):
        mock_abstract_local_query_execute_query.return_value = Response(
            status=HTTP_200_OK
        )

        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_get(
            data_rest_views.ExecuteLocalQueryView.as_view(),
            mock_user,
            data={"query": "{}"},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestDataLocalQueryPostPermissions(SimpleTestCase):
    @patch.object(data_api, "execute_query")
    def test_anonymous_returns_http_200(self, mock_data_api_execute_query):
        mock_data_api_execute_query.return_value = []

        response = RequestMock.do_request_post(
            data_rest_views.ExecuteLocalQueryView.as_view(), None, data={"query": "{}"}
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(data_api, "execute_query")
    def test_authenticated_returns_http_200(self, mock_data_api_execute_query):
        mock_data_api_execute_query.return_value = []

        mock_user = create_mock_user("1")

        response = RequestMock.do_request_post(
            data_rest_views.ExecuteLocalQueryView.as_view(),
            mock_user,
            data={"query": "{}"},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(data_api, "execute_query")
    def test_staff_returns_http_200(self, mock_data_api_execute_query):
        mock_data_api_execute_query.return_value = []

        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_post(
            data_rest_views.ExecuteLocalQueryView.as_view(),
            mock_user,
            data={"query": "{}"},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestDataKeywordQueryGetPermissions(SimpleTestCase):
    @patch.object(AbstractExecuteLocalQueryView, "execute_query")
    def test_anonymous_returns_http_200(self, mock_abstract_local_query_execute_query):
        mock_abstract_local_query_execute_query.return_value = Response(
            status=HTTP_200_OK
        )

        response = RequestMock.do_request_get(
            data_rest_views.ExecuteLocalQueryView.as_view(), None, data={"query": "{}"}
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(AbstractExecuteLocalQueryView, "execute_query")
    def test_authenticated_returns_http_200(
        self, mock_abstract_local_query_execute_query
    ):
        mock_abstract_local_query_execute_query.return_value = Response(
            status=HTTP_200_OK
        )

        mock_user = create_mock_user("1")

        response = RequestMock.do_request_get(
            data_rest_views.ExecuteLocalQueryView.as_view(),
            mock_user,
            data={"query": "{}"},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(AbstractExecuteLocalQueryView, "execute_query")
    def test_staff_returns_http_200(self, mock_abstract_local_query_execute_query):
        mock_abstract_local_query_execute_query.return_value = Response(
            status=HTTP_200_OK
        )

        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_get(
            data_rest_views.ExecuteLocalQueryView.as_view(),
            mock_user,
            data={"query": "{}"},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestDataKeywordQueryPostPermissions(SimpleTestCase):
    @patch.object(data_api, "execute_query")
    def test_anonymous_returns_http_200(self, mock_data_api_execute_query):
        mock_data_api_execute_query.return_value = []

        response = RequestMock.do_request_post(
            data_rest_views.ExecuteLocalKeywordQueryView.as_view(),
            None,
            data={"query": "{}"},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(data_api, "execute_query")
    def test_authenticated_returns_http_200(self, mock_data_api_execute_query):
        mock_data_api_execute_query.return_value = []

        mock_user = create_mock_user("1")

        response = RequestMock.do_request_post(
            data_rest_views.ExecuteLocalKeywordQueryView.as_view(),
            mock_user,
            data={"query": "{}"},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(data_api, "execute_query")
    def test_staff_returns_http_200(self, mock_data_api_execute_query):
        mock_data_api_execute_query.return_value = []

        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_post(
            data_rest_views.ExecuteLocalKeywordQueryView.as_view(),
            mock_user,
            data={"query": "{}"},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestDataAssignPatchPermissions(SimpleTestCase):
    @patch.object(data_api, "assign")
    @patch.object(workspace_api, "get_by_id")
    @patch.object(data_api, "get_by_id")
    def test_anonymous_returns_http_403(
        self,
        mock_data_api_get_by_id,
        mock_workspace_api_get_by_id,
        mock_data_api_assign,
    ):
        mock_data_api_get_by_id.return_value = None
        mock_workspace_api_get_by_id.return_value = None
        mock_data_api_assign.return_value = None

        response = RequestMock.do_request_patch(
            data_rest_views.DataAssign.as_view(),
            None,
            param={"pk": 0, "workspace_id": 0},
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(data_api, "assign")
    @patch.object(workspace_api, "get_by_id")
    @patch.object(data_api, "get_by_id")
    def test_authenticated_returns_http_200(
        self,
        mock_data_api_get_by_id,
        mock_workspace_api_get_by_id,
        mock_data_api_assign,
    ):
        mock_data_api_get_by_id.return_value = None
        mock_workspace_api_get_by_id.return_value = None
        mock_data_api_assign.return_value = None

        mock_user = create_mock_user("1")

        response = RequestMock.do_request_patch(
            data_rest_views.DataAssign.as_view(),
            mock_user,
            param={"pk": 0, "workspace_id": 0},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(data_api, "assign")
    @patch.object(workspace_api, "get_by_id")
    @patch.object(data_api, "get_by_id")
    def test_staff_returns_http_200(
        self,
        mock_data_api_get_by_id,
        mock_workspace_api_get_by_id,
        mock_data_api_assign,
    ):
        mock_data_api_get_by_id.return_value = None
        mock_workspace_api_get_by_id.return_value = None
        mock_data_api_assign.return_value = None

        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_patch(
            data_rest_views.DataAssign.as_view(),
            mock_user,
            param={"pk": 0, "workspace_id": 0},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestDataChangeOwnerPatchPermissions(SimpleTestCase):
    @patch.object(data_api, "change_owner")
    @patch("core_main_app.components.user.api.get_user_by_id")
    @patch.object(Data, "get_by_id")
    def test_anonymous_returns_http_403(
        self,
        mock_data_api_get_by_id,
        mock_user_api_get_by_id,
        mock_data_api_change_owner,
    ):
        mock_data_api_get_by_id.return_value = None
        mock_user_api_get_by_id.return_value = None
        mock_data_api_change_owner.return_value = None

        response = RequestMock.do_request_patch(
            data_rest_views.DataChangeOwner.as_view(),
            None,
            param={"pk": 0, "user_id": 0},
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(data_api, "change_owner")
    @patch("core_main_app.components.user.api.get_user_by_id")
    @patch.object(Data, "get_by_id")
    def test_authenticated_returns_http_403(
        self,
        mock_data_api_get_by_id,
        mock_user_api_get_by_id,
        mock_data_api_change_owner,
    ):
        mock_data_api_get_by_id.return_value = None
        mock_user_api_get_by_id.return_value = None
        mock_data_api_change_owner.return_value = None

        mock_user = create_mock_user("1")

        response = RequestMock.do_request_patch(
            data_rest_views.DataChangeOwner.as_view(),
            mock_user,
            param={"pk": 0, "user_id": 0},
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(data_api, "change_owner")
    @patch("core_main_app.components.user.api.get_user_by_id")
    @patch.object(Data, "get_by_id")
    def test_staff_returns_http_200(
        self,
        mock_data_api_get_by_id,
        mock_user_api_get_by_id,
        mock_data_api_change_owner,
    ):
        # Arrange
        # is_staff to access the view
        # is_superuser to be able to change the owner
        user_request = create_mock_user("1", is_staff=True, is_superuser=True)
        mock_data_api_get_by_id.return_value = None
        mock_user_api_get_by_id.return_value = None
        mock_data_api_change_owner.return_value = None

        # Act
        response = RequestMock.do_request_patch(
            data_rest_views.DataChangeOwner.as_view(),
            user_request,
            param={"pk": 0, "user_id": 0},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestDataPermissions(SimpleTestCase):
    @patch.object(Data, "get_by_id")
    def test_superuser_returns_http_200(self, get_by_id):
        mock_user = create_mock_user("1", is_superuser=True)
        mock_data = Data(user_id="1")
        get_by_id.return_value = mock_data

        response = RequestMock.do_request_get(
            data_rest_views.DataPermissions.as_view(),
            mock_user,
            data={"ids": f'["{mock_data.id}"]'},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(Data, "get_by_id")
    def test_authenticated_returns_http_200(self, get_by_id):
        mock_user = create_mock_user("1")
        mock_data = Data(user_id="1")
        get_by_id.return_value = mock_data

        response = RequestMock.do_request_get(
            data_rest_views.DataPermissions.as_view(),
            mock_user,
            data={"ids": f'["{mock_data.id}"]'},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(Data, "get_by_id")
    def test_staff_returns_http_200(self, get_by_id):
        mock_user = create_mock_user("1", is_staff=True)
        mock_data = Data(user_id="1")
        get_by_id.return_value = mock_data

        response = RequestMock.do_request_get(
            data_rest_views.DataPermissions.as_view(),
            mock_user,
            data={"ids": f'["{mock_data.id}"]'},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(Data, "get_by_id")
    def test_get_returns_200_for_anonymous(self, get_by_id):
        # Arrange
        mock_user = create_mock_user("999", is_anonymous=True)
        mock_data = Data(user_id="1")
        get_by_id.return_value = mock_data

        # Mock
        response = RequestMock.do_request_get(
            data_rest_views.DataPermissions.as_view(), mock_user, data={"ids": '["1"]'}
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestDataMigrationPermission(SimpleTestCase):
    def setUp(self):
        self.fake_id = "507f1f77bcf86cd799439011"

    def test_anonymous_validation_returns_http_403(self):
        # Arrange
        mock_user = create_mock_user("1", is_anonymous=True)

        # Act
        response = RequestMock.do_request_post(
            data_validation.as_view(),
            mock_user,
            param={"pk": self.fake_id},
            data={"data": f'["{self.fake_id}"]'},
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_staff_validation_returns_http_403(self):
        # Arrange
        request_user = create_mock_user("1", is_staff=True)

        # Act
        response = RequestMock.do_request_post(
            data_validation.as_view(),
            request_user,
            param={"pk": self.fake_id},
            data={"data": f'["{self.fake_id}"]'},
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(data_api, "migrate_data_list")
    def test_admin_validation_returns_http_200(self, migration):
        # Arrange
        migration.return_value = "123"
        request_user = create_mock_user("1", is_superuser=True)

        # Act
        response = RequestMock.do_request_post(
            data_validation.as_view(),
            request_user,
            param={"pk": self.fake_id},
            data={"data": f'["{self.fake_id}"]'},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_anonymous_migration_returns_http_403(self):
        # Arrange
        mock_user = create_mock_user("1", is_anonymous=True)

        # Act
        response = RequestMock.do_request_post(
            data_migration.as_view(),
            mock_user,
            param={"pk": self.fake_id},
            data={"data": f'["{self.fake_id}"]'},
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_staff_migration_returns_http_403(self):
        # Arrange
        request_user = create_mock_user("1", is_staff=True)

        # Act
        response = RequestMock.do_request_post(
            data_migration.as_view(),
            request_user,
            param={"pk": self.fake_id},
            data={"data": f'["{self.fake_id}"]'},
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(data_api, "migrate_data_list")
    def test_admin_migration_returns_http_200(self, migration):
        # Arrange
        migration.return_value = "123"
        request_user = create_mock_user("1", is_superuser=True)

        # Act
        response = RequestMock.do_request_post(
            data_migration.as_view(),
            request_user,
            param={"pk": self.fake_id},
            data={"data": f'["{self.fake_id}"]'},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)
